import pendulum
import typer

import pprint

from app.auth import Auth
from app.logs import Log
from app.settings import Settings
from app.web import Web
from rich.console import Console
from rich.table import Table
from rich.text import Text
from rich import print


settings = Settings.config()
if "loglevel" in settings["vikunja-cli"]["general"]:
    log = Log("List", settings["vikunja-cli"]["general"]["loglevel"])
else:
    log = Log("List")

auth = Auth()
app = typer.Typer()
console = Console()
jwt = auth.get_jwt()


@app.command()
def list():
    r = Web.request(token=jwt, method="GET", path="/lists")
    lists = r.json()
    table = Table("ID", "Title", "Description", "Owner", "Created at")
    for list in lists:
        is_archived = ""
        is_favorite = ""
        if list["is_archived"]:
            is_archived = emoji.emojize(":file_cabinet:")
        if list["is_favorite"]:
            is_favorite = emoji.emojize(":star:")
        created = pendulum.parse(list["created"])
        created = created.format("dddd Do MMMM YYYY HH:mm:ss")
        table.add_row(
            f"{list['id']}",
            f"{is_archived}{is_favorite} [#{list['hex_color']}]{list['title']}[/]",
            f"{list['description']}",
            f"{list['owner']['username']}",
            f"{created}",
        )

    console.print(table)


@app.command()
def set_default(
    name: str = typer.Argument(None),
    list_id: int = typer.Option(None, help="List ID where you want to add this task"),
):
    if name is None and list_id is None:
        print("[bold red]You need to specify a list name or a --list-id[/]")
        raise typer.Exit(code=1)
    if name:
        r = Web.request(token=jwt, method="GET", path="/lists", payload={"s": name})
        list = r.json()
        if not list:
            print(f'[bold red]List "{name}" doesn\'t exist[/]')
            raise typer.Exit(code=1)
        else:
            if len(list) > 1:
                print(
                    f'[bold red]Many list with the name "{name}". Specify id with --list-id[/]'
                )
                raise typer.Exit(code=1)
            else:
                list_id = list[0]["id"]
    elif list_id:
        if list_id.isdigit():
            list_id = list_id
        else:
            print(
                "[bold red]Bravo, you have successfully passed a non int into an int.[/]"
            )
    settings["vikunja-cli"]["general"]["default_list"] = list_id
    if Settings.save(settings):
        print(f"[bold green]Default list is now {name}[/]")
    else:
        print(f"[bold red]Can't save new configuration[/]")


if __name__ == "__main__":
    app()
