import os
import re
import requests
import typer
import yaml

import pprint

from app.logs import Log
from pathlib import Path

log = Log("Settings")

home = str(Path.home())


class Settings:

    @staticmethod
    def init():
        config = {"vikunja-cli": {"general": {}, "auth": {}}}
        url = typer.prompt(
            "What is the URL of your Vikunja API ? (https://vikunja.tld/api/v1)"
        )
        if not Settings._url_validator(url):
            typer.echo(f"{url} is not a valid URL")
            raise typer.Exit(code=1)
        try:
            r = requests.get(f"{url}/info")
            r.raise_for_status()
        except equests.exceptions.RequestException as err:
            f"[{Messages.error()}] Your vikunja instance doesn't anwser with url {url}"
            raise typer.Exit(code=1)

        config["vikunja-cli"]["general"]["url"] = url
        user = typer.prompt("What is your username ?")
        password = typer.prompt("What is your password ?")
        totp_enabled = typer.confirm("TOTP is enabled ?")
        totp_passcode = ""
        if totp_enabled:
            totp_passcode = typer.prompt(
                "What is your TOTP access token (if enabled) ?"
            )
        config["vikunja-cli"]["auth"] = {
            "username": user,
            "password": password,
            "totp_passcode": totp_passcode,
        }
        Settings.save(config)

    @staticmethod
    def config():
        if not os.path.isfile(f"{home}/.config/vikunja-cli/config.yaml"):
            typer.echo(
                "First launch ? Yeah ! We will ask you some questions to configure vikunja-cli"
            )
            typer.run(Settings.init)
        settings = {}
        try:
            with open(f"{home}/.config/vikunja-cli/config.yaml", "r") as config_file:
                config = yaml.safe_load(config_file)
        except (OSError, ValueError, BaseException, yaml.YAMLError):
            log.critical("Failed to load configuration !")
            raise typer.Exit(code=1)

        return config

    @staticmethod
    def save(config):
        try:
            Path(f"{home}/.config/vikunja-cli/").mkdir(parents=True, exist_ok=True)
        except OSError:
            log.critical(f"Can't create {home}/.config/vikunja-cli !")
            raise typer.Exit(code=1)
        try:
            with open(f"{home}/.config/vikunja-cli/config.yaml", "w") as config_file:
                yaml.dump(config, config_file, default_flow_style=False)
        except (OSError, ValueError, BaseException, yaml.YAMLError):
            log.critical(
                f"Can't write config file {home}/.config/vikunja-cli/config.yaml !"
            )
            raise typer.Exit(code=1)
        return True

    @staticmethod
    def _url_validator(url):
        regex = re.compile(
            r"^(?:http|ftp)s?://"  # http:// or https://
            r"(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|"  # domain...
            r"localhost|"  # localhost...
            r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"  # ...or ip
            r"(?::\d+)?"  # optional port
            r"(?:/?|[/?]\S+)$",
            re.IGNORECASE,
        )
        checker = re.match(regex, url)
        return checker
